/*****************************************************************************/
/*                                                                           */
/* Modul: MYDIAL.H                                                           */
/* Datum: 04/05/91                                                           */
/*                                                                           */
/*****************************************************************************/

#ifndef __MYDIAL__
#define __MYDIAL__

/****** DEFINES **************************************************************/

/* Erweiterte Objekttypen */
#define DIALMOVER  0x0011
#define DCRBUTTON  0x0012
#define UNDERLINE  0x0013
#define TITLEBOX   0x0014
#define HELPBUT    0x0015
#define CIRCLEBUT  0x0016
#define POPUPSTRG  0x0017
#define KPOPUPSTRG 0x0018
#define SLIDEPART  0x0019
#define LONGINPUT  0x001A
#define UNDOBUT    0x001F

#define FLAGS11    0x0800	/* fr mit UNDO Taste bedienbares Objekt */
#define FLAGS15    0x8000	/* mehrzeiliges Eingabeobjekt */

/* Flags fr popup_select */
#define	DO_POPUP		 0
#define	DO_CYCLE		-2
#define	DO_POPPREV	-1
#define	DO_POPNEXT	 1

/* Zentrierflags fr Alertboxtext */
#define ALCENTER   "\001"
#define ALRIGHT    "\002"

#define	MAXASTRINGS	15		/* Maximale Anzahl von Alertzeilen (incl. 3 Buttons) */
#define	MAXASLENGTH	50		/* Maximale Lnge einer Alertzeile */

/****** TYPES ****************************************************************/

typedef struct
{
  UWORD e_flags;
  UWORD e_bclk;
  UWORD e_bmsk;
  UWORD e_bst;
  UWORD e_m1flags;
  GRECT e_m1;
  UWORD e_m2flags;
  GRECT e_m2;
  WORD FAR *e_mepbuf;
  WORD  e_ltime;
  WORD  e_htime;
  WORD  e_mx;       
  WORD  e_my;
  WORD  e_mb;
  WORD  e_ks;
  UWORD e_kr;
  WORD  e_br;
} MYEVENT;

typedef struct
{
	LONG cookie;			/* mu 'VSCR' sein */
	LONG product;			/* Analog zur XBRA-Kennung */
	WORD version;			/* Version des VSCR-Protokolls, zunchst 0x100 */
	WORD x, y, w, h;	/* Sichtbarer Ausschnitt des Bildschirms */
} INFOVSCR;

typedef struct
{
	OBJECT 	*Tree;
	MFDB 	Buffer;
	WORD 	x, y, w, h;
} DIALINFO;

typedef struct
{
	USERBLK	ublk;
	UWORD		old_type;
} OBBLK;

typedef VOID (*FORMUSREVENT) (MYEVENT *evdata);
typedef VOID (*FORMEVNTFUNC) (WORD *event, MYEVENT *evdata);

typedef WORD (*FORMKEYFUNC) (OBJECT *, WORD, WORD, WORD, WORD, WORD *, UWORD *);

typedef VOID (*FIXOBJFUNC) (OBJECT *tree, WORD obj);


/****** VARIABLES ************************************************************/

extern INFOVSCR *infovscr;


/****** FUNCTIONS ************************************************************/

/* Daten des Systemfonts ermitteln und Systemfont frs Programm einstellen */
BOOLEAN get_sysfnt   _((WORD vdi_handle, WORD *font_id, WORD *gl_point, WORD *gl_wchar, WORD *gl_hchar));

/* Cookie suchen; bei TRUE wurde Cookie gefunden */
BOOLEAN getcookie    _((LONG cookie, LONG *p_value));

/* Ist ein VSCR Cookie vorhanden und ist der sicht-  */
/* bare Bildschirm kleiner als der Gesamtbildschirm? */
BOOLEAN ask_vscr     _((VOID));


/*****************************************************************************/

/* VDI-Farbindizes fr dreidimensionale Dialoge einstellen                   */
/* back:  Hintergrund (default: 8/hellgrau)                                  */
/* light: helle ("beleuchtete") Kanten (default: 0/Wei)                     */
/* dark:  dunkle Kanten (default: 9/Dunkelgrau)                              */
VOID		dial_3Dcolors  (WORD back, WORD light, WORD dark);

/* Darstellung von dreidimensionalen ROOT-Objekten (Bumen) beeinflussen     */
/* border: Randstrke des ROOT-Objektes in Pixeln (inkl. perimt)             */
/* perimt: TRUE = auch einen schwarzen, 1 Pixel starken Rand zeichen         */
VOID		dial_3Droot    (BOOLEAN border, BOOLEAN perimt);

/* Nach der Cursorposition in Pixeln das Zeichen eines Eingabefeldes         */
/* berechnen, auf dem der Cursor steht; wichtig fr 3D-Editobjekte           */
WORD dial_charpos     (OBJECT *tree, WORD obj, WORD curspos, WORD wbox);

/* Gegenstck zu dial_charpos: Pixelposition des Cursors abhngig von dem    */
/* Zeichen, auf dem er steht, berechnen;  wichtig fr 3D-Editobjekte         */
WORD dial_curspos     (OBJECT *tree, WORD obj, WORD charpos, WORD wbox);

/* Cursor auf ein bestimmtes Zeichen eines Objekts setzen                    */
WORD dial_setcpos     (OBJECT *tree, WORD obj, WORD *idx);

/* Cursorposition ermitteln (Editfeld, Buchstabenposition) */
VOID dial_getcpos     (WORD *obj, WORD *idx);

/* Editfeld-Cursor mit Exklusiv-Oder zeichnen (= ein- bzw. ausschalten) */
VOID dial_drawcursor  (OBJECT *tree, WORD edit_obj, WORD char_idx, WORD whandle);


/*****************************************************************************/

/* Parentobjekt ermitteln; bei ROOT wird NIL (-1) zurckgegeben */
WORD    parent       _((OBJECT *tree, WORD obj));

/* ob_spec ermitteln (auch fr USERDEF Objekte) */
LONG    get_obspec   _((OBJECT *tree, WORD obj));

/* ob_spec ndern (auch fr USERDEF Objekte) */
VOID    set_obspec   _((OBJECT *tree, WORD obj, LONG obspec));

/* evnt_multi kstate und kreturn mit NKCC in normalisierten KeyCode wandeln */
UWORD   normkey      _((WORD ks, WORD kr));


/*****************************************************************************/

/* Trgt eine eigene Funktion ein, die aufgerufen wird, wenn der erweiterte  */
/* Objekttyp eines Objekts den MyDials unbekannt ist.                        */
VOID set_fixobj       _((FIXOBJFUNC fun));

/* Liefert einen Zeiger auf die benutzerdefinierbare Funktion */
FIXOBJFUNC get_fixobj _((VOID));

/* USERBLK fr ein Objekt erzeugen, danach Objekt in Userdef wandeln */
USERBLK *add_ublock   _((WORD cdecl (*code)(PARMBLK *parmblock), LONG obspec, WORD type));


/*****************************************************************************/

/* entspricht form_do: Dialog ausfhren */
WORD    dial_do      _((DIALINFO *di, WORD edit_obj));

/* Dialog verschieben; x,y,w,h geben Begrenzungsrechteck an: i.d.R. Desktop ohne Menzeile */
/* dial_move wird auch von dial_do aufgerufen */
VOID    dial_move    _((DIALINFO *di, WORD x, WORD y, WORD w, WORD h));

/* DIALINFO Struktur mit Werten fllen, Hintergrund buffern */
BOOLEAN dial_start   _((OBJECT *tree, DIALINFO *di));

/* Hintergrund restaurieren, Speicher freigeben */
VOID    dial_end     _((DIALINFO *di));

/* Dialog zentrieren; entspricht form_center */
VOID    dial_center  _((OBJECT *tree));

/* Dialog zeichnen; entspricht objc_draw */
VOID    dial_draw    _((DIALINFO *di));



/*****************************************************************************/

/* Nchste Zeile eines mehrzeiligen Eingabefeldes suchen */
WORD    find_next    _((OBJECT *tree, WORD obj));

/* Vorherige Zeile eines mehrzeiligen Eingabefeldes suchen */
WORD    find_prev    _((OBJECT *tree, WORD obj));

WORD    get_idx      _((OBJECT *tree, WORD obj, WORD cpos));

/* ersetzt form_keybd, ruft mit set_Keybd definierte User Tastaturroutine mit gleichen Parametern auf */
/* Parameter siehe form_keybd */
WORD    form_Keybd   _((OBJECT *tree, WORD edit_obj, WORD next_obj, WORD kr, WORD ks, WORD *onext_obj, UWORD *okr));

WORD    form_Button  _((WORD handle, OBJECT *tree, WORD obj, WORD clicks, WORD *nextobj));

/* Shortcut im Objektbaum suchen; liefert Objekt oder NIL               */
/* startobj: Ast, dessen gesammte Unterobjekte auf Shortcut untersucht  */
/*           werden sollen. Normalerweise ROOT (Ausnahme: Popups)       */
/* ks, kr  : Rckgabewerte von evnt_multi, die den Shortcut beschreiben */
WORD    shortcut     _((OBJECT *tree, WORD startobj, WORD ks, WORD kr));

/* Eigene Event-Auswertungsroutine in dial_do einklinken */
/* dial_do kann auch um neue Events erweitert werden (z.B. Timerevent)  */
/* Die Routine wird nach dem Beenden des Dialoges wieder ausgeklinkt    */
VOID    set_MyEvnt   _((FORMUSREVENT init_func, FORMEVNTFUNC evnt_func));

/* Adresse einer vom User eingeklinkten Event-Auswertungsroutine ermitteln */
VOID    get_MyEvnt   _((FORMUSREVENT *init_func, FORMEVNTFUNC *evnt_func));

/* Eigene Tastaturroutine in form_Keybd einklinken */
/* Wenn die eigene Routine die Taste nicht auswerten konnte, mu sie NIL zurckgeben, */
/* ansonsten den Rckgabewert von form_button oder form_keybd. */
/* Die Routine wird nach dem Beenden des Dialoges wieder ausgeklinkt */
VOID    set_Keybd    _((FORMKEYFUNC fun));

/* Adresse einer vom User eingeklinkten Tastaturroutine ermitteln */
FORMKEYFUNC get_Keybd _((VOID));

/* Internes MyDial objc_edit.                         */
/* Wenn der Dialog im Fenster liegt, mu winhndl die  */
/* Fensterkennung enthalten, sonst NIL. Dadurch wird  */
/* der Cursor ber die Rechteckliste gezeichnet, um   */
/* Redraw-Fehler zu vermeiden.                        */
WORD		obj_edit		_((OBJECT *ob_edtree, WORD ob_edobject, WORD kstate, WORD ob_edchar, WORD *ob_edidx, WORD ob_edkind, WORD mode, WORD *next_obj, WORD winhndl));

/* Einen String in ein Objekt kopieren                             */
/* Diese Funktion arbeitet auch korrekt mit MyDial-Userdefs.       */
/* Folgende Objekt-Typen werden untersttzt:                       */
/* G_BUTTON, G_STRING, G_TITLE, G_CICON, G_ICON, G_BOXCHAR         */
/* G_TEXT, G_BOXTEXT G_FTEXT, G_FBOXTEXT (jeweils te_ptext)        */
/* Rckgabewert: Ein Zeiger auf den Zielstring                     */
BYTE *objc_setstring _((OBJECT *tree, WORD obj, CONST BYTE *text));

/* Den Text eines Objekts in einen String kopieren                 */
/* Diese Funktion arbeitet auch korrekt mit MyDial-Userdefs.       */
/* Folgende Objekt-Typen werden untersttzt:                       */
/* G_BUTTON, G_STRING, G_TITLE, G_CICON, G_ICON, G_BOXCHAR         */
/* G_TEXT, G_BOXTEXT G_FTEXT, G_FBOXTEXT (jeweils te_ptext)        */
/* Rckgabewert: Ein Zeiger auf den Zielstring                     */
BYTE *objc_getstring _((OBJECT *tree, WORD obj, BYTE *text));

/*****************************************************************************/

/* entspricht form_alert, ruft dial_alert mit dial_alert (NULL, string, defbut, 0, ALI_LEFT) auf. */
WORD    do_alert     _((WORD defbut, CONST BYTE *txt));

/* erweiterter form_alert Aufruf: */
/* alicon : eigenes Icon oder Objekt verwenden; bei NULL wird die Iconangabe im String verwendet */
/* string : Alertstring, wie in form_alert */
/*          Wenn der Text eines Buttons mit einem Punkt beginnt, wird er als default Button verwendet, */
/*          beginnt er mit einem Doppelpunkt, ist es der Default-Undobutton. */
/*          In diesen beiden Fllen werden die Werte von defbut und undobut ignoriert. */
/* defbut : Default Exitbutton */
/* undobut: Default Undobutton, wird bei Druck auf UNDO Taste selektiert */
/* align  : Positionieren des Alerttextes: ALI_LEFT, ALI_CENTER, ALI_RIGHT, in VDI.H definiert */
/*          Einzelne Zeilen knnen durch voranstellen von ALCENTER oder ALRIGHT anders positioniert werden */
WORD    dial_alert   _((OBJECT *alicon, CONST BYTE *string, WORD defbut, WORD undobut, WORD align));

/* Liefert zu einer Iconnummer den Zeiger auf die dazu gehrige BITBLK Struktur */
BITBLK  *get_icon    _((WORD icon));

/* Zerlegt Alertstring in 8 einzelne Strings, 5 Textzeilen und 3 Buttons */
/* Gibt Iconnummer zurck */
WORD    al_token     _((CONST BYTE *f_alert, BYTE str[8][50], WORD *txtnum, WORD *butnum));

/* Gre einer Alertbox berechnen */
/* alicon  : eigenes Icon oder Objekt verwenden; bei NULL wird die Iconangabe im String verwendet */
/* string  : Alertstring, wie in form_alert                                                       */
/* defbut  : Default Exitbutton                                                                   */
/* undobut : Default Undobutton                                                                   */
/* tree    : wenn ungleich NULL, dann enthlt es nach dem Aufruf einen Zeiger auf den Alertbaum   */
/* firstbut: wenn ungleich NULL, dann enthlt es nach dem Aufruf die Nummer des ersten Buttons    */
/* alert_texte: wenn ungleich NULL, dann enthlt es nach dem Aufruf alle Alerttexte               */
/*              Das Array hat eine Gre von alert_texte[MAXASTRINGS][MAXASLENGTH]                */
/* size    : Enthlt nach dem Aufruf die Breite und Hhe der Alertbox                             */
/* al_dat  : Wenn ungleich NULL, dann Zeiger auf 9 WORDs, die Daten der Alertbox enthalten        */
WORD alert_data      _((OBJECT *alicon, CONST BYTE *string, WORD defbut, WORD undobut, OBJECT **tree, WORD *firstbut, BYTE alert_texte[][MAXASLENGTH], RECT *size, WORD al_dat[]));

/* Alertbox zeichnen */
/* alicon : eigenes Icon oder Objekt verwenden; bei NULL wird die Iconangabe im String verwendet     */
/* alert_texte: Enthlt die Texte des Alerts. Wird von alert_data initialisiert.                     */
/* size   : Enthlt die Breite und Hhe der Alertbox. Wenn handle ungleich NIL, dann wird            */
/*          size->x und size->y fr die Positionierung der Alertbox ausgewertet.                     */
/* al_dat : Zeiger auf 9 WORDs, die Daten der Alertbox enthalten. Wird von alert_data initialisiert. */
/* align  : Positionieren des Alerttextes (siehe dial_alert)                                         */
/* clip   : Wenn ungleich NULL, dann wird die Alertbox ber dieses Clipping-Rechteck gezeichnet      */
VOID alert_draw      _((OBJECT *alicon, BYTE alert_texte[][MAXASLENGTH], RECT *size, WORD al_dat[], WORD align, RECT *clip));

/* Alertbox beenden */
/* close_alert initialisiert den Alert-Baum wieder, damit er sich wieder im Urzustand befindet.      */
/* Diese Funktion mu nur bei Verwendung von draw_alert benutzt werden.                              */
/* al_dat : Zeiger auf 9 WORDs, die Daten der Alertbox enthalten. Wird von alert_data initialisiert. */
VOID alert_close     _((WORD al_dat[]));

/******************************************************************************/

/* popup_menu: Popupmen anzeigen															*/
/* tree      : Adresse des Objektbaums, in dem sich das Popupmen befindet		*/
/* obj       : index des Popups																*/
/* x, y      : Position der linken oberen Popup-Ecke (bzw.des Objektzentrums, */
/*             wenn center_obj != NIL)														*/
/* center_obj: enthlt den Index des zu zentrierenden Popup-Eintrags.			*/
/* relative  : TRUE: Popup wird an der Mausposition gezeichnet, x und y			*/
/*             sollten dann 0 sein.															*/
/* bmsk      : gibt den Mausbutton an, mit dem das Popup verlassen wird			*/
/*             (1 = links, 2 = rechts, wie beim AES)									*/
/* ok        : TRUE, wenn Hintergund gesichert werden konnte, FALSE sonst		*/
/* Ergebnis  : ausgewhlter Eintrag oder NIL												*/

WORD    popup_menu   _((OBJECT *tree, WORD obj, WORD x, WORD y, WORD center_obj, BOOLEAN relative, WORD bmsk, BOOLEAN *ok));

/* popup_select: Popup-Button in Dialog mit Cycle-Button verwalten */
/* dialtree  : Dialog-Baum																		*/
/* btn       : Index des Objekts, da das Popup aufruft (erlaubte Typen: BoxText, Button, String, Userdef-Boxtext oder Userdef-Button) */
/* poptree   : Adresse des Objektbaums, in dem sich das Popupmen befindet. */
/* obj       : Index des Popups																*/
/*             Die einzelnen Eintrge mssen aus Strings oder tastaturbedienbaren Strings (Userdef 19) bestehen. */
/* docheck   : aktiven Popup-Eintrag (mit Button-Text) mit Haken versehen		*/
/* docycle   : DO_POPUP: Popup anzeigen; DO_CYCLE, DO_POPNEXT: nchsten Ein-	*/
/*             trag darstellen; DO_POPPREV: vorherigen Eintrag darstellen		*/
/* wh        : Fensterhandle. Wenn wh=NIL, dann kein Redraw ber Fensterhandle */
/* ok        : TRUE, wenn Hintergund gesichert werden konnte, FALSE sonst		*/
/* Ergebnis  : ausgewhlter Eintrag oder NIL												*/

WORD	   popup_select _((WORD wh, OBJECT *dialtree, WORD btn, OBJECT *poptree, WORD obj, BOOLEAN docheck, WORD docycle, BOOLEAN *ok));



/*****************************************************************************/

/* Dialog ffnen, fhrt Growbox aus, und ruft dial_start auf */
/* bei size = NULL ist der Ausgangspunkt der Growbox die Bildschirmmitte */
/* Rckgabewert gibt an, ob Hintergrund gebuffert werden konnte */
BOOLEAN open_dial    _((OBJECT *tree, BOOLEAN grow, RECT *size, DIALINFO *dialinfo));

/* Dialog schlieen, fhrt Shrinkbox aus, und ruft dial_end auf */
/* bei size = NULL ist der Zielpunkt der Shrinkbox die Bildschirmmitte */
BOOLEAN close_dial   _((BOOLEAN shrink, RECT *size, DIALINFO *dialinfo));

/* Komplette Dialogabarbeitung, ruft open_dial, dial_do und close_dial auf */
/* ok gibt an, ob Hintergrund gebuffert werden konnte */
WORD    HndlDial     _((OBJECT *tree, WORD def, BOOLEAN grow_shrink, RECT *size, BOOLEAN *ok));



/*****************************************************************************/

/* Initialisieren von Sliderobjekten in einer Resource. Es wird nur eine Positios-  */
/* anpassung durchgefhrt, damit sich Objekte mit einem Rand von 1 Pixel auerhalb  */
/* nicht berlappen. bergabeparameter sind die Objektindizes der einzelnen Slider- */
/* elemente, fr nicht vorhandene Objekte wird NIL bergeben. Allerdings mssen bei */
/* bergebenen Sliderboxen auch die Pfeile vorhanden sein. Die 'slbox'-Objekte sind */
/* der gesamte Sliderbereich, nicht nur der verschiebbare Teil. */
VOID    dial_slinit   (OBJECT *tree, WORD closer, WORD head, WORD edge,
                              WORD vslbox, WORD up, WORD down, WORD hslbox, WORD left, WORD right);

/* trgt USERDEF Objekte ein, is_dialog gibt an, ob es eine Dialogbox oder ein Pulldownmen ist */
VOID    dial_fix     _((OBJECT  *tree, BOOLEAN is_dialog));

/* Initialisieren der Mydials. Am Programmstart 1 mal aufrufen */
/* nkc_init und nkc_set werden in dial_init NICHT aufgerufen, mu vom Hauptprogramm gemacht werden */
BOOLEAN dial_init    _((VOID *alc, VOID *fr, WORD *mnum, MFORM **mform, BOOLEAN *gs, BOOLEAN do3D, OBBLK *block, WORD blocklen));

/* Deinitialisieren der mydials. Am Programmende 1 mal aufrufen */
/* nkc_exit wird in dial_exit NICHT aufgerufen, mu vom Hauptprogramm gemacht werden */
BOOLEAN dial_exit    _((VOID));


/*****************************************************************************/

/* Lscht das GEM-Clipboard */
WORD    scrap_clear  _((VOID));

/* ndert die berschrift der ASCII-Tabelle. Gedacht fr Sprachanpassung, */
/* da die MyDial-Resource nicht zugnglich ist.                           */
/* header mu auf einen String zeigen. Die MyDials ndern nur den Pointer,*/
/* daher mu der String dauerhaft angelegt werden.                        */
VOID    ascii_head   _((BYTE *header));


/*****************************************************************************/

/* Shortcut in einem Menbaum suchen                    */
/* menu  : Men-Baum                                    */
/* mk    : Struktur, die die gedrckte Taste beschreibt */
/* title : der Mentitel von "item"                     */
/* item  : Meneintrag, zu dem der Shortcut pat        */
/* Ergebnis: TRUE = Shortcurt gefunden, sonst FALSE     */

BOOLEAN my_menu_key  _((OBJECT *menu, MKINFO *mk, WORD *title, WORD *item));

#endif /* __MYDIAL__ */
